@@ -47,12 +47,15 @@
 #' The default multiplicity adjustment method is \code{"tukey"}, which is only
 #' approximate when the standard errors differ.
 #'
-#' \code{poly.emmc} generates orthogonal polynomial contrasts, assuming
-#' equally-spaced factor levels. These are derived from the
+#' \code{poly.emmc} and \code{opoly.emmc} generate orthogonal polynomial contrasts.
+#' \code{poly.emmc} uses equally-spaced factor levels; coefficients are derived from the
 #' \code{\link[stats]{poly}} function, but an \emph{ad hoc} algorithm is used to
 #' scale them to integer coefficients that are (usually) the same as in
-#' published tables of orthogonal polynomial contrasts. The default multiplicity
-#' adjustment method is \code{"none"}.
+#' published tables of orthogonal polynomial contrasts. On the other hand, 
+#' \code{opoly.emmc}'s coefficients are always normalized (sum of squares equals 1),
+#' but allows the user to choose alternate reference points in \code{scores}, as in the
+#' \code{\link{contr.poly}} function.
+#' In both cases, the default multiplicity adjustment method is \code{"none"}.
 #'
 #' \code{trt.vs.ctrl.emmc} and its relatives generate contrasts for comparing
 #' one level (or the average over specified levels) with each of the other
@@ -82,6 +85,19 @@
 #' respectively, as in subtracting the overall EMM from each EMM. The default
 #' multiplicity adjustment method is \code{"fdr"}. This is a Bonferroni-based
 #' method and is slightly conservative; see \code{\link[stats]{p.adjust}}.
+#' 
+#' \code{nrmlz.emmc} is a wrapper that can be used with any other \code{.emmc} 
+#' function that will normalize the contrast coefficients so that the sum of its 
+#' squares equals 1. Just provide the root name of the function in \code{family},
+#' along with any other arguments to pass to it.
+#' 
+#' \code{wtcon.emmc} generates weighted contrasts based on the function 
+#' \code{\link[multcomp]{contrMat}} function in the \pkg{multcomp} package,
+#' using the provided \code{type} as documented there. If the user provides 
+#' \code{wts}, they have to conform to the length of \code{levs}; however,
+#' if \code{wts} is not specified, \code{contrast} will fill-in what is
+#' required, and usually this is safer (especially when \code{by != NULL}
+#' which usually means that the weights are different in each \code{by} group).
 #'
 #' \code{identity.emmc} simply returns the identity matrix (as a data frame),
 #' minus any columns specified in \code{exclude}. It is potentially useful in
@@ -131,19 +147,24 @@
 #' pairs(warp.emm, exclude = 2)
 #' # (same results using exclude = "M" or include = c("L","H") or include = c(1,3))
 #'
+#'# Same contrasts as above but with normalized contrast coefficients
+#'contrast(warp.emm, "nrmlz", family = "pairwise", include = c(1, 3))
+#'
 #' ### Setting up a custom contrast function
-#' helmert.emmc <- function(levs, ...) {
-#'     M <- as.data.frame(contr.helmert(levs))
-#'     names(M) <- paste(levs[-1],"vs earlier")
-#'     attr(M, "desc") <- "Helmert contrasts"
+#' revhelmert.emmc <- function(levs, ...) {
+#'     M <- as.data.frame(contr.helmert(levs)[rev(seq_along(levs)), ])
+#'     names(M) <- paste(rev(levs)[-1],"vs later")
+#'     attr(M, "desc") <- "reverse Helmert contrasts"
 #'     M
 #' }
-#' contrast(warp.emm, "helmert")
+#' contrast(warp.emm, "revhelmert")
+#' 
 #' \dontrun{
 #' # See what is used for polynomial contrasts with 6 levels
 #' emmeans:::poly.emmc(1:6)
 #' }
 #' @name contrast-methods
+#' @export
 pairwise.emmc = function(levs, exclude = integer(0), include, ...) {
     exclude = .get.excl(levs, exclude, include)
     k = length(levs)
@@ -170,6 +191,7 @@ pairwise.emmc = function(levs, exclude = integer(0), include, ...) {
 
 # all pairwise trt[j] - trt[i], j > i
 #' @rdname emmc-functions
+#' @export
 revpairwise.emmc = function(levs, exclude = integer(0), include, ...) {
     exclude = .get.excl(levs, exclude, include)
     k = length(levs)
@@ -196,6 +218,7 @@ revpairwise.emmc = function(levs, exclude = integer(0), include, ...) {
 # pseudonym
 #' @rdname emmc-functions
 #' @param reverse Logical value to determine the direction of comparisons
+#' @export
 tukey.emmc = function(levs, reverse = FALSE, ...) {
     if (reverse)
         revpairwise.emmc(levs, ...)
@@ -207,6 +230,7 @@ tukey.emmc = function(levs, reverse = FALSE, ...) {
 # ad hoc scaling works for up to 13 levels
 #' @rdname emmc-functions
 #' @param max.degree Integer specifying the maximum degree of polynomial contrasts
+#' @export
 poly.emmc = function(levs, max.degree = min(6, k-1), ...) {
     nm = c("linear", "quadratic", "cubic", "quartic", paste("degree",5:20))
     k = length(levs)
@@ -229,12 +253,53 @@ poly.emmc = function(levs, max.degree = min(6, k-1), ...) {
     M
 }
 
+# Orthonormal poly contrasts - not rescaled
+#' @rdname emmc-functions
+#' @param scores Set of values of length \code{length(levs)} over which
+#'   orthogonal polynomials are computed. The default scores are the 
+#'   consecutive integers \code{seq_along(levs)}.
+#'   (If \code{exclude} or \code{include}
+#'   are used, the default scores are subsetted accordingly; and if \code{scores} 
+#'   is specified, its length must be the same as that of the subsetted \code{levs}).
+#' @export
+opoly.emmc = function(levs, max.degree = min(6, k-1), scores, 
+                      exclude = integer(0), include, ...) {
+    if (MS <- missing(scores))
+        scores = seq_along(levs)
+    exclude = .get.excl(levs, exclude, include)
+    if(length(exclude) > 0) {
+        oldlevs = levs
+        if (MS)
+            scores = scores[-exclude]
+        levs = levs[-exclude]
+    }
+    nm = c("linear", "quadratic", "cubic", "quartic", paste("degree",5:20))
+    k = length(levs)
+    if (length(scores) != k)
+        stop("In opoly.emmc: Lengths of 'scores' must equal ", length(levs), call. = FALSE)
+    M = contr.poly(k, contrasts = TRUE, scores = scores, sparse = FALSE)
+    if(length(exclude) > 0) {
+        MM = M
+        levs = oldlevs
+        M = matrix(0, nrow = length(oldlevs), ncol = ncol(MM))
+        M[-exclude, ] = MM
+    }
+    M = as.data.frame(M[, seq_len(min(k-1, max.degree)), drop = FALSE])
+    row.names(M) = levs
+    names(M) = nm[seq_len(ncol(M))]
+    attr(M, "desc") = "normalized polynomial contrasts"
+    attr(M, "adjust") = "none"
+    M
+}
+
+
 # All comparisons with a control; ref = index of control group
 # New version -- allows more than one control group (ref is a vector)
 #' @rdname emmc-functions
 #' @param ref Integer(s) or character(s) specifying which level(s) to use 
 #'   as the reference. Character values must exactly match elements of \code{levs}
 #'   (including any enhancements -- see examples)
+#' @export
 trt.vs.ctrl.emmc = function(levs, ref = 1, reverse = FALSE, 
                             exclude = integer(0), include, ...) {
     ref = .num.key(levs, ref)
@@ -274,18 +339,21 @@ trt.vs.ctrl.emmc = function(levs, ref = 1, reverse = FALSE,
 
 # control is 1st level
 #' @rdname emmc-functions
+#' @export
 trt.vs.ctrl1.emmc = function(levs, ref = 1, ...) {
     trt.vs.ctrl.emmc(levs, ref = ref, ...)
 }
 
 # control is last level
 #' @rdname emmc-functions
+#' @export
 trt.vs.ctrlk.emmc = function(levs, ref = length(levs), ...) {
     trt.vs.ctrl.emmc(levs, ref = ref, ...)
 }
 
 # pseudonym for trt.vs.ctrl
 #' @rdname emmc-functions
+#' @export
 dunnett.emmc = function(levs, ref = 1, ...) {
     trt.vs.ctrl.emmc(levs, ref = ref, ...)
 }
@@ -300,6 +368,7 @@ dunnett.emmc = function(levs, ref = 1, ...) {
 #'   In the former case, weights for any excluded levels are set to zero.
 #'   \code{wts} has no impact on the results unless there are at least
 #'   three levels included in the contrast.
+#' @export
 eff.emmc = function(levs, exclude = integer(0), include, wts = rep(1, length(levs)), ...) {
     exclude = .get.excl(levs, exclude, include)
     if ((length(exclude) > 0) && (length(wts) == length(levs) - length(exclude))) {
@@ -328,6 +397,7 @@ eff.emmc = function(levs, exclude = integer(0), include, wts = rep(1, length(lev
 }
 
 #' @rdname emmc-functions
+#' @export
 del.eff.emmc = function(levs, exclude = integer(0), include, wts = rep(1, length(levs)), ...) {
     M = eff.emmc(levs, exclude, include, wts, ...)
     use = setdiff(seq_along(levs), .get.excl(levs, exclude, include))
@@ -340,6 +410,7 @@ del.eff.emmc = function(levs, exclude = integer(0), include, wts = rep(1, length
 # Contrasts to compare consecutive levels:
 # (-1,1,0,0,...), (0,-1,1,0,...), ..., (0,...0,-1,1)
 #' @rdname emmc-functions
+#' @export
 consec.emmc = function(levs, reverse = FALSE, exclude = integer(0), include, ...) {
     exclude = .get.excl(levs, exclude, include)
     sgn = ifelse(reverse, -1, 1)
@@ -370,6 +441,7 @@ consec.emmc = function(levs, reverse = FALSE, exclude = integer(0), include, ...
 # Mean after minus mean before
 # e.g., (-1, 1/3,1/3,1/3), (-1/2,-1/2, 1/2,1/2), (-1/3,-1/3,-1/3, 1)
 #' @rdname emmc-functions
+#' @export
 mean_chg.emmc = function(levs, reverse = FALSE, exclude = integer(0), include, ...) {
     exclude = .get.excl(levs, exclude, include)
     sgn = ifelse(reverse, -1, 1)
@@ -394,8 +466,57 @@ mean_chg.emmc = function(levs, reverse = FALSE, exclude = integer(0), include, .
     M
 }
 
+#' @rdname emmc-functions
+#' @export
+helmert.emmc <- function(levs, exclude = integer(0), include, ...) {
+    exclude = .get.excl(levs, exclude, include)
+    k = length(levs) - length(exclude)
+    M = contr.helmert(k)
+    if (length(exclude) > 0) {
+        M = matrix(0, nrow = length(levs), ncol = k-1)
+        M[-exclude, ] = contr.helmert(k)
+        lbl = levs[-exclude]
+    }
+    else
+        lbl = levs
+    M <- as.data.frame(M)
+    names(M) <- paste(lbl[-1],"vs earlier")
+    attr(M, "desc") <- "Helmert contrasts"
+    attr(M, "adjust") = "none"
+    if(length(exclude) > 0)
+        attr(M, "famSize") = k
+    M
+}
+
+#' @rdname emmc-functions
+#' @param family name of contrast family to use
+#' @export
+nrmlz.emmc = function(levs, family, ...) {
+    M = get(paste0(family, ".emmc"))(levs, ...)
+    for (i in seq_len(ncol(M)))
+        M[, i] = M[, i] / sqrt(sum(M[, i]^2))
+    attr(M, "desc") = paste("Normalized", attr(M, "desc"))
+    M
+}
+
+# weighted contrasts
+#' @rdname emmc-functions
+#' @param cmtype the \code{type} argument passed to \code{\link[multcomp]{contrMat}}
+#' @export
+wtcon.emmc = function(levs, wts = rep(1, length(levs)), cmtype = "GrandMean", ...) {
+    if (!requireNamespace("multcomp"))
+        stop("The 'multcomp' package must be installed to use 'wtcon' contrasts", call. = FALSE)
+    
+    names(wts) = levs
+    M = data.frame(t(multcomp::contrMat(wts, type = cmtype, ...)))
+    M
+}
+
+
+
 # Non-contrasts -- just pass thru, possibly excluding some levels
 #' @rdname emmc-functions
+#' @export
 identity.emmc = function(levs, exclude = integer(0), include, ...) {
     exclude = .get.excl(levs, exclude, include)
     k = length(levs) - length(exclude)
@@ -410,7 +531,12 @@ identity.emmc = function(levs, exclude = integer(0), include, ...) {
 }
 
 
+
 ### utility to translate character keys to index keys
+#' @rdname extending-emmeans
+#' @order 43
+#' @param levs,key The \code{.num.key} function returns the numeric indices of
+#' the levels in \code{levs} to the set of all levels in \code{key}
 #' @export
 .num.key = function(levs, key) {
     if(!is.null(raw <- attr(levs, "raw")))
@@ -428,6 +554,12 @@ identity.emmc = function(levs, exclude = integer(0), include, ...) {
 
 ### utility to find exclude levels from either exclude or include
 ### Also returns numeric version
+#' @rdname extending-emmeans
+#' @order 34
+#' @param exc,inc Arguments for \code{.get.excl} which is useful
+#' in writing \code{.emmc} functions for generating contrast coefficients,
+#' and supports arguments \code{exclude} or \code{include} for excluding
+#' or specifying which levels to use.
 #' @export
 .get.excl = function(levs, exc, inc) {
     if (!missing(inc)) {
