@@ -85,6 +85,11 @@ str.emmGrid <- function(object, ...) {
         cat(paste(tmp, collapse = ", "))
         cat("\n")
     }
+    if (length(cf <- object@roles$counterfactuals) > 0) {
+        cat("Counterfactuals\n    ")
+        cat(paste(cf, collapse = ", "))
+        cat("\n")
+    }
     if(!is.null(object@model.info$nesting)) {
         cat("Nesting structure:  ")
         cat(.fmt.nest(object@model.info$nesting))
@@ -103,19 +108,27 @@ str.emmGrid <- function(object, ...) {
 
 
 #' @rdname emmGrid-methods
-#' @param export Logical value. If \code{FALSE}, the object is printed. 
+#' @param export Logical value. If \code{FALSE}, the object is printed.
 #'   If \code{TRUE}, a list is invisibly returned, which contains character
-#'   elements named \code{summary} and \code{annotations} that may be saved 
-#'   or displayed as the user sees fit. \code{summary} is a character matrix 
+#'   elements named \code{summary} and \code{annotations} that may be saved
+#'   or displayed as the user sees fit. \code{summary} is a character matrix
 #'   (or list of such matrices, if a \code{by} variable is in effect).
-#'   \code{annotations} is a character vector of the annotations that would 
+#'   \code{annotations} is a character vector of the annotations that would
 #'   have been printed below the summary or summaries.
 #' @method print emmGrid
 #' @param x An \code{emmGrid} object
 #' @export
-print.emmGrid = function(x, ..., export = FALSE)
+print.emmGrid = function(x, ..., export = FALSE) {
     print(summary.emmGrid(x, ...), export = export)
+    # if(export)
+    #     print(summary.emmGrid(x, ...), export = export)
+    # else
+    #     show(x)
+}
 
+### Former print method which I changed to work around a bug in pkgdown
+# print.emmGrid = function(x, ..., export = FALSE)
+#     print(summary.emmGrid(x, ...), export = export)
 
 # vcov method
 #' Miscellaneous methods for \code{emmGrid} objects
@@ -160,6 +173,18 @@ vcov.emmGrid = function(object, ..., sep = get_emm_option("sep")) {
 }
 
 
+#' @rdname emmGrid-methods
+#'
+#' @returns The \code{linfct} function and method returns the \code{linfct} slot of \code{object}.
+#' @export
+linfct = function(object, ...)
+    UseMethod("linfct")
+
+#' @rdname emmGrid-methods
+#' @export
+linfct.default = function(object, ...) {
+    attr(object, "linfct")
+}
 
 
 # Method to alter contents of misc slot
@@ -911,7 +936,8 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     
     if (is.na(object@post.beta[1]) && !missing(N.sim)) {
         message("Simulating a sample of size ", N.sim, " of regression coefficients.")
-        object@post.beta = sim(N.sim, object@bhat, object@V)
+        bh = object@bhat[!is.na(object@bhat)]
+        object@post.beta = sim(N.sim, bh, object@V)
     }
 
     if (transform == "pass")
@@ -921,65 +947,58 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     if ((transform == "response") && (!is.null(object@misc$tran2)))
              object = regrid(object, transform = "mu")
 
-    .collapse = (\(.collapse = NULL, ...) .collapse)(...) # check counterfactuals hook
-    
     # Save post.beta stuff
     PB = object@post.beta
     NC = attr(PB, "n.chains")
     
     if (!is.na(PB[1])) { # fix up post.beta BEFORE we overwrite parameters
-        PB = PB %*% t(object@linfct)
+        PB = PB %*% t(object@linfct[ , !is.na(object@bhat), drop = FALSE])
         if (".offset." %in% names(object@grid))
             PB = t(apply(PB, 1, function(.) . + object@grid[[".offset."]]))
     }
     
     est = .est.se.df(object, do.se = TRUE) ###FALSE)
     
-    if(is.null(.collapse)) {
-        estble = !(is.na(est[[1]]))
-        object@V = vcov(object)[estble, estble, drop = FALSE]
-        object@bhat = est[[1]]
-        object@linfct = diag(1, length(estble))
-        object@misc$regrid.flag = TRUE
-        pargs = object@grid[names(object@levels)]
-        lbls = do.call(paste, c(pargs, sep = "."))
-        if (!is.null(disp <- object@misc$display)) {  # fix up for the bookkeeping in nested models
-            object@V = object@V[estble, estble, drop = FALSE]
-            object@linfct = matrix(0, nrow = length(disp), ncol = length(estble))
-            object@linfct[disp, ] = diag(1, length(estble))
-            lbls = lbls[disp]
-        }
-        colnames(object@linfct) = lbls
-        if(all(estble))
-            object@nbasis = estimability::all.estble
-        else
-            object@nbasis = object@linfct[, !estble, drop = FALSE]
-        
-        # override the df function
-        df = est$df
-        edf = df[estble]
-        if (length(edf) == 0) edf = NA
-        # note both NA/NA and Inf/Inf test is.na() = TRUE
-        prev.df.msg = attr(object@dffun, "mesg")
-        if (any(is.na(edf/edf)) || (diff(range(edf)) < .01)) { # use common value
-            object@dfargs = list(df = mean(edf, na.rm = TRUE))
-            object@dffun = function(k, dfargs) dfargs$df
-        }
-        else { # use containment df
-            object@dfargs = list(df = df)
-            object@dffun = function(k, dfargs) {
-                idx = which(zapsmall(k) != 0)
-                ifelse(length(idx) == 0, NA, min(dfargs$df[idx], na.rm = TRUE))
-            }
-        }
-        if(!is.null(prev.df.msg)) 
-            attr(object@dffun, "mesg") = ifelse(
-                startsWith(prev.df.msg, "inherited"), prev.df.msg,
-                paste("inherited from", prev.df.msg, "when re-gridding"))
+    estble = !(is.na(est[[1]]))
+    object@V = vcov(object)[estble, estble, drop = FALSE]
+    object@bhat = est[[1]]
+    object@linfct = diag(1, length(estble))
+    object@misc$regrid.flag = TRUE
+    pargs = object@grid[names(object@levels)]
+    lbls = do.call(paste, c(pargs, sep = "."))
+    if (!is.null(disp <- object@misc$display)) {  # fix up for the bookkeeping in nested models
+        object@V = object@V[estble, estble, drop = FALSE]
+        object@linfct = matrix(0, nrow = length(disp), ncol = length(estble))
+        object@linfct[disp, ] = diag(1, length(estble))
+        lbls = lbls[disp]
     }
+    colnames(object@linfct) = lbls
+    if(all(estble))
+        object@nbasis = estimability::all.estble
+    else
+        object@nbasis = object@linfct[, !estble, drop = FALSE]
     
-    if(!is.null(.collapse) && is.null(object@misc$tran))  # need explicit link so we can collapse
-        object@misc$tran = attr(est, "link") = make.link("identity")
+    # override the df function
+    df = est$df
+    edf = df[estble]
+    if (length(edf) == 0) edf = NA
+    # note both NA/NA and Inf/Inf test is.na() = TRUE
+    prev.df.msg = attr(object@dffun, "mesg")
+    if (any(is.na(edf/edf)) || (diff(range(edf)) < .01)) { # use common value
+        object@dfargs = list(df = mean(edf, na.rm = TRUE))
+        object@dffun = function(k, dfargs) dfargs$df
+    }
+    else { # use containment df
+        object@dfargs = list(df = df)
+        object@dffun = function(k, dfargs) {
+            idx = which(zapsmall(k) != 0)
+            ifelse(length(idx) == 0, NA, min(dfargs$df[idx], na.rm = TRUE))
+        }
+    }
+    if(!is.null(prev.df.msg)) 
+        attr(object@dffun, "mesg") = ifelse(
+            startsWith(prev.df.msg, "inherited"), prev.df.msg,
+            paste("inherited from", prev.df.msg, "when re-gridding"))
     
     if(transform %in% c("response", "mu", "unlink", links, "user") && !is.null(object@misc$tran)) {
         flink = link = attr(est, "link")
@@ -995,36 +1014,11 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
         }
         if (!is.na(PB[1]))
             PB = matrix(flink$linkinv(PB), ncol = ncol(PB))
-        if(is.null(.collapse)) {
-            D = flink$mu.eta(object@bhat[estble])
-            object@bhat = flink$linkinv(object@bhat)
-            # efficient repl for D'VD with D <- diag(D)
-            object@V = sweep(sweep(object@V, 1, D, "*"), 2, D, "*")
-        }
-        else {  # we'll average over the levels of .collapse (assume it varies slowest)
-            est = est[[1]]
-            nobs = length(object@levels[[.collapse]])
-            idx = sapply(seq_len(nobs), \(i) which(object@grid[[.collapse]] == i))
-            X = sweep(object@linfct, 1, flink$mu.eta(est), "*")
-            wt.counter = (\(wt.counter, ...) wt.counter)(...)
-            if (length(wt.counter) == 1) wt.counter = rep(1, nobs)
-            wmn = function(x) sum(wt.counter*x) / sum(wt.counter)
-            est = flink$linkinv(est)
-            object@bhat = sapply(seq_len(nrow(idx)), \(i) wmn(est[idx[i,]]))
-            L = matrix(0, nrow = nrow(idx), ncol = ncol(X))
-            for (i in seq_len(nrow(idx)))
-                L[i, ] = apply(X[idx[i,], , drop = FALSE], 2, wmn)
-            object@V = L %*% tcrossprod(object@V, L)
-            object@linfct = diag(1, nrow(L))
-            object@misc$regrid.flag = TRUE
-            if(!is.na(PB[1])) {
-                pb = matrix(0, ncol = nrow(L), nrow = nrow(PB))
-                for (i in seq_len(nrow(idx)))
-                    pb[, i] = apply(PB[, idx[i,], drop = FALSE], 1, wmn)
-                PB = pb
-            }
-        }
-        
+        D = flink$mu.eta(object@bhat[estble])
+        object@bhat = flink$linkinv(object@bhat)
+        # efficient repl for D'VD with D <- diag(D)
+        object@V = sweep(sweep(object@V, 1, D, "*"), 2, D, "*")
+
         inm = object@misc$inv.lbl
         if (!is.null(inm)) {
             object@misc$estName = inm
@@ -1072,6 +1066,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     }
     
     if(!is.na(PB[1])) {
+        PB = PB[ , !is.na(object@bhat), drop = FALSE]
         attr(PB, "n.chains") = NC
         object@post.beta = PB
     }
@@ -1082,13 +1077,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     object@model.info$model.matrix = "Submodels are not available with regridded objects"
     if(!missing(predict.type))
         object = update(object, predict.type = predict.type)
-    if(!is.null(.collapse)) {
-        object@grid = object@grid[idx[,1], , drop = TRUE]
-        object@grid[[.collapse]] = object@levels[[.collapse]] = NULL
-        object@roles$predictors = setdiff(object@roles$predictors, .collapse)
-        object@misc$famSize = nrow(object@linfct)
-    }
-    
+
     object
 }
 
